# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: ethermint/types/v1/account.proto, ethermint/types/v1/dynamic_fee.proto, ethermint/types/v1/indexer.proto, ethermint/types/v1/web3.proto
# plugin: python-betterproto
# This file has been @generated
from dataclasses import dataclass

import betterproto

from ....cosmos.auth import v1beta1 as ___cosmos_auth_v1_beta1__


@dataclass(eq=False, repr=False)
class TxResult(betterproto.Message):
    """TxResult is the value stored in eth tx indexer"""

    height: int = betterproto.int64_field(1)
    """the block height"""

    tx_index: int = betterproto.uint32_field(2)
    """cosmos tx index"""

    msg_index: int = betterproto.uint32_field(3)
    """the msg index in a batch tx"""

    eth_tx_index: int = betterproto.int32_field(4)
    """
    eth tx index, the index in the list of valid eth tx in the block,
    aka. the transaction list returned by eth_getBlock api.
    """

    failed: bool = betterproto.bool_field(5)
    """if the eth tx is failed"""

    gas_used: int = betterproto.uint64_field(6)
    """
    gas used by tx, if exceeds block gas limit,
    it's set to gas limit which is what's actually deducted by ante handler.
    """

    cumulative_gas_used: int = betterproto.uint64_field(7)
    """the cumulative gas used within current batch tx"""


@dataclass(eq=False, repr=False)
class ExtensionOptionsWeb3Tx(betterproto.Message):
    typed_data_chain_id: int = betterproto.uint64_field(1)
    """
    typed data chain id used only in EIP712 Domain and should match
    Ethereum network ID in a Web3 provider (e.g. Metamask).
    """

    fee_payer: str = betterproto.string_field(2)
    """
    fee payer is an account address for the fee payer. It will be validated
    during EIP712 signature checking.
    """

    fee_payer_sig: bytes = betterproto.bytes_field(3)
    """
    fee payer sig is a signature data from the fee paying account,
    allows to perform fee delegation when using EIP712 Domain.
    """


@dataclass(eq=False, repr=False)
class ExtensionOptionDynamicFeeTx(betterproto.Message):
    """
    ExtensionOptionDynamicFeeTx is an extension option that specify the maxPrioPrice
    for cosmos tx
    """

    max_priority_price: str = betterproto.string_field(1)
    """the same as `max_priority_fee_per_gas` in eip-1559 spec"""


@dataclass(eq=False, repr=False)
class EthAccount(betterproto.Message):
    """
    EthAccount implements the authtypes.AccountI interface and embeds an
    authtypes.BaseAccount type. It is compatible with the auth AccountKeeper.
    """

    base_account: '___cosmos_auth_v1_beta1__.BaseAccount' = betterproto.message_field(1)
    code_hash: str = betterproto.string_field(2)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: ibc/lightclients/localhost/v1/localhost.proto
# plugin: python-betterproto
# This file has been @generated
from dataclasses import dataclass

import betterproto

from ....core.client import v1 as ___core_client_v1__


@dataclass(eq=False, repr=False)
class ClientState(betterproto.Message):
    """
    ClientState defines a loopback (localhost) client. It requires (read-only)
    access to keys outside the client prefix.
    """

    chain_id: str = betterproto.string_field(1)
    """self chain ID"""

    height: '___core_client_v1__.Height' = betterproto.message_field(2)
    """self latest block height"""



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: ibc/lightclients/solomachine/v1/solomachine.proto
# plugin: python-betterproto
# This file has been @generated
from dataclasses import dataclass

import betterproto
import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf

from ....core.channel import v1 as ___core_channel_v1__
from ....core.connection import v1 as ___core_connection_v1__


class DataType(betterproto.Enum):
    """
    DataType defines the type of solo machine proof being created. This is done
    to preserve uniqueness of different data sign byte encodings.
    """

    DATA_TYPE_UNINITIALIZED_UNSPECIFIED = 0
    """Default State"""

    DATA_TYPE_CLIENT_STATE = 1
    """Data type for client state verification"""

    DATA_TYPE_CONSENSUS_STATE = 2
    """Data type for consensus state verification"""

    DATA_TYPE_CONNECTION_STATE = 3
    """Data type for connection state verification"""

    DATA_TYPE_CHANNEL_STATE = 4
    """Data type for channel state verification"""

    DATA_TYPE_PACKET_COMMITMENT = 5
    """Data type for packet commitment verification"""

    DATA_TYPE_PACKET_ACKNOWLEDGEMENT = 6
    """Data type for packet acknowledgement verification"""

    DATA_TYPE_PACKET_RECEIPT_ABSENCE = 7
    """Data type for packet receipt absence verification"""

    DATA_TYPE_NEXT_SEQUENCE_RECV = 8
    """Data type for next sequence recv verification"""

    DATA_TYPE_HEADER = 9
    """Data type for header verification"""


@dataclass(eq=False, repr=False)
class ClientState(betterproto.Message):
    """
    ClientState defines a solo machine client that tracks the current consensus
    state and if the client is frozen.
    """

    sequence: int = betterproto.uint64_field(1)
    """latest sequence of the client state"""

    frozen_sequence: int = betterproto.uint64_field(2)
    """frozen sequence of the solo machine"""

    consensus_state: 'ConsensusState' = betterproto.message_field(3)
    allow_update_after_proposal: bool = betterproto.bool_field(4)
    """
    when set to true, will allow governance to update a solo machine client.
    The client will be unfrozen if it is frozen.
    """


@dataclass(eq=False, repr=False)
class ConsensusState(betterproto.Message):
    """
    ConsensusState defines a solo machine consensus state. The sequence of a
    consensus state is contained in the "height" key used in storing the
    consensus state.
    """

    public_key: 'betterproto_lib_google_protobuf.Any' = betterproto.message_field(1)
    """public key of the solo machine"""

    diversifier: str = betterproto.string_field(2)
    """
    diversifier allows the same public key to be re-used across different solo
    machine clients (potentially on different chains) without being considered
    misbehaviour.
    """

    timestamp: int = betterproto.uint64_field(3)


@dataclass(eq=False, repr=False)
class Header(betterproto.Message):
    """Header defines a solo machine consensus header"""

    sequence: int = betterproto.uint64_field(1)
    """sequence to update solo machine public key at"""

    timestamp: int = betterproto.uint64_field(2)
    signature: bytes = betterproto.bytes_field(3)
    new_public_key: 'betterproto_lib_google_protobuf.Any' = betterproto.message_field(4)
    new_diversifier: str = betterproto.string_field(5)


@dataclass(eq=False, repr=False)
class Misbehaviour(betterproto.Message):
    """
    Misbehaviour defines misbehaviour for a solo machine which consists
    of a sequence and two signatures over different messages at that sequence.
    """

    client_id: str = betterproto.string_field(1)
    sequence: int = betterproto.uint64_field(2)
    signature_one: 'SignatureAndData' = betterproto.message_field(3)
    signature_two: 'SignatureAndData' = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class SignatureAndData(betterproto.Message):
    """
    SignatureAndData contains a signature and the data signed over to create that
    signature.
    """

    signature: bytes = betterproto.bytes_field(1)
    data_type: 'DataType' = betterproto.enum_field(2)
    data: bytes = betterproto.bytes_field(3)
    timestamp: int = betterproto.uint64_field(4)


@dataclass(eq=False, repr=False)
class TimestampedSignatureData(betterproto.Message):
    """
    TimestampedSignatureData contains the signature data and the timestamp of the
    signature.
    """

    signature_data: bytes = betterproto.bytes_field(1)
    timestamp: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class SignBytes(betterproto.Message):
    """SignBytes defines the signed bytes used for signature verification."""

    sequence: int = betterproto.uint64_field(1)
    timestamp: int = betterproto.uint64_field(2)
    diversifier: str = betterproto.string_field(3)
    data_type: 'DataType' = betterproto.enum_field(4)
    """type of the data used"""

    data: bytes = betterproto.bytes_field(5)
    """marshaled data"""


@dataclass(eq=False, repr=False)
class HeaderData(betterproto.Message):
    """HeaderData returns the SignBytes data for update verification."""

    new_pub_key: 'betterproto_lib_google_protobuf.Any' = betterproto.message_field(1)
    """header public key"""

    new_diversifier: str = betterproto.string_field(2)
    """header diversifier"""


@dataclass(eq=False, repr=False)
class ClientStateData(betterproto.Message):
    """
    ClientStateData returns the SignBytes data for client state verification.
    """

    path: bytes = betterproto.bytes_field(1)
    client_state: 'betterproto_lib_google_protobuf.Any' = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class ConsensusStateData(betterproto.Message):
    """
    ConsensusStateData returns the SignBytes data for consensus state
    verification.
    """

    path: bytes = betterproto.bytes_field(1)
    consensus_state: 'betterproto_lib_google_protobuf.Any' = betterproto.message_field(
        2
    )


@dataclass(eq=False, repr=False)
class ConnectionStateData(betterproto.Message):
    """
    ConnectionStateData returns the SignBytes data for connection state
    verification.
    """

    path: bytes = betterproto.bytes_field(1)
    connection: '___core_connection_v1__.ConnectionEnd' = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class ChannelStateData(betterproto.Message):
    """
    ChannelStateData returns the SignBytes data for channel state
    verification.
    """

    path: bytes = betterproto.bytes_field(1)
    channel: '___core_channel_v1__.Channel' = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class PacketCommitmentData(betterproto.Message):
    """
    PacketCommitmentData returns the SignBytes data for packet commitment
    verification.
    """

    path: bytes = betterproto.bytes_field(1)
    commitment: bytes = betterproto.bytes_field(2)


@dataclass(eq=False, repr=False)
class PacketAcknowledgementData(betterproto.Message):
    """
    PacketAcknowledgementData returns the SignBytes data for acknowledgement
    verification.
    """

    path: bytes = betterproto.bytes_field(1)
    acknowledgement: bytes = betterproto.bytes_field(2)


@dataclass(eq=False, repr=False)
class PacketReceiptAbsenceData(betterproto.Message):
    """
    PacketReceiptAbsenceData returns the SignBytes data for
    packet receipt absence verification.
    """

    path: bytes = betterproto.bytes_field(1)


@dataclass(eq=False, repr=False)
class NextSequenceRecvData(betterproto.Message):
    """
    NextSequenceRecvData returns the SignBytes data for verification of the next
    sequence to be received.
    """

    path: bytes = betterproto.bytes_field(1)
    next_seq_recv: int = betterproto.uint64_field(2)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: evmos/epochs/v1/genesis.proto, evmos/epochs/v1/query.proto
# plugin: python-betterproto
# This file has been @generated
from dataclasses import dataclass
from datetime import (
    datetime,
    timedelta,
)
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase

from ....cosmos.base.query import v1beta1 as ___cosmos_base_query_v1_beta1__


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class EpochInfo(betterproto.Message):
    identifier: str = betterproto.string_field(1)
    start_time: datetime = betterproto.message_field(2)
    duration: timedelta = betterproto.message_field(3)
    current_epoch: int = betterproto.int64_field(4)
    current_epoch_start_time: datetime = betterproto.message_field(5)
    epoch_counting_started: bool = betterproto.bool_field(6)
    current_epoch_start_height: int = betterproto.int64_field(7)


@dataclass(eq=False, repr=False)
class GenesisState(betterproto.Message):
    """GenesisState defines the epochs module's genesis state."""

    epochs: List['EpochInfo'] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class QueryEpochsInfoRequest(betterproto.Message):
    pagination: '___cosmos_base_query_v1_beta1__.PageRequest' = (
        betterproto.message_field(1)
    )


@dataclass(eq=False, repr=False)
class QueryEpochsInfoResponse(betterproto.Message):
    epochs: List['EpochInfo'] = betterproto.message_field(1)
    pagination: '___cosmos_base_query_v1_beta1__.PageResponse' = (
        betterproto.message_field(2)
    )


@dataclass(eq=False, repr=False)
class QueryCurrentEpochRequest(betterproto.Message):
    identifier: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class QueryCurrentEpochResponse(betterproto.Message):
    current_epoch: int = betterproto.int64_field(1)


class QueryStub(betterproto.ServiceStub):
    async def epoch_infos(
        self,
        query_epochs_info_request: 'QueryEpochsInfoRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryEpochsInfoResponse':
        return await self._unary_unary(
            '/evmos.epochs.v1.Query/EpochInfos',
            query_epochs_info_request,
            QueryEpochsInfoResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def current_epoch(
        self,
        query_current_epoch_request: 'QueryCurrentEpochRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryCurrentEpochResponse':
        return await self._unary_unary(
            '/evmos.epochs.v1.Query/CurrentEpoch',
            query_current_epoch_request,
            QueryCurrentEpochResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class QueryBase(ServiceBase):
    async def epoch_infos(
        self, query_epochs_info_request: 'QueryEpochsInfoRequest'
    ) -> 'QueryEpochsInfoResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def current_epoch(
        self, query_current_epoch_request: 'QueryCurrentEpochRequest'
    ) -> 'QueryCurrentEpochResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_epoch_infos(
        self,
        stream: 'grpclib.server.Stream[QueryEpochsInfoRequest, QueryEpochsInfoResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.epoch_infos(request)
        await stream.send_message(response)

    async def __rpc_current_epoch(
        self,
        stream: 'grpclib.server.Stream[QueryCurrentEpochRequest, QueryCurrentEpochResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.current_epoch(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            '/evmos.epochs.v1.Query/EpochInfos': grpclib.const.Handler(
                self.__rpc_epoch_infos,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryEpochsInfoRequest,
                QueryEpochsInfoResponse,
            ),
            '/evmos.epochs.v1.Query/CurrentEpoch': grpclib.const.Handler(
                self.__rpc_current_epoch,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryCurrentEpochRequest,
                QueryCurrentEpochResponse,
            ),
        }



from __future__ import annotations

from dataclasses import dataclass
from typing import Any, Mapping, Sequence, TypedDict


class WithValidator(TypedDict):
    """:class:`~typing.TypedDict` with string ``validator_address`` field."""

    validator_address: str
    """Validator address."""


class MsgWithValidatorInterface(TypedDict):
    """Validator editing message."""

    type: str  # noqa: A003
    """Type for ABI encoding."""
    value: WithValidator
    """Message itself."""


class MsgInterface(TypedDict):
    """Validator editing message."""

    type: str  # noqa: A003
    """Type for ABI encoding."""
    value: Any
    """Message itself."""


@dataclass
class Domain:
    """This describes ``domain`` field of :class:`EIPToSign`."""

    name: str
    """Domain name."""
    version: str
    """Version (usually ``1.0.0``)."""
    chainId: int  # noqa: N815
    """Chain ID."""
    verifyingContract: str  # noqa: N815
    """Verifying contract address."""
    salt: str
    """Used salt (usually ``'0'``)."""


@dataclass
class EIPToSign:
    """EIP message to sign."""

    types: dict[str, Any]
    """Message types for ABI encoding, mapping name to type."""
    primaryType: str  # noqa: N815
    """Type name of ``message`` attribute."""
    domain: Domain
    """Domain."""
    message: dict[str, Any]
    """Message to sign itself, mapping."""


def create_eip712(
    types: dict[str, Any], chain_id: int, message: dict[str, Any]
) -> EIPToSign:
    """Create `EIP712 <https://eips.ethereum.org/EIPS/eip-712>`_ data."""
    return EIPToSign(
        types=types,
        primaryType='Tx',
        domain=Domain(
            name='Cosmos Web3',
            version='1.0.0',
            chainId=chain_id,
            verifyingContract='cosmos',
            salt='0',
        ),
        message=message,
    )


def generate_message_with_multiple_transactions(
    account_number: str,
    sequence: str,
    chain_cosmos_id: str,
    memo: str,
    fee: dict[str, Any],
    msgs: Sequence[Mapping[str, Any]],
) -> dict[str, Any]:
    """Create a message with multiple transactions included."""
    return {
        'account_number': account_number,
        'chain_id': chain_cosmos_id,
        'fee': fee,
        'memo': memo,
        'msgs': msgs,
        'sequence': sequence,
    }


def generate_message(
    account_number: str,
    sequence: str,
    chain_cosmos_id: str,
    memo: str,
    fee: dict[str, Any],
    msg: Mapping[str, Any],
) -> dict[str, Any]:
    """Create a message with one transaction included."""
    return generate_message_with_multiple_transactions(
        account_number,
        sequence,
        chain_cosmos_id,
        memo,
        fee,
        [msg],
    )


def generate_types(msg_values: dict[str, Any]) -> dict[str, Any]:
    """Generate EIP-712 types."""
    types = {
        'EIP712Domain': [
            {'name': 'name', 'type': 'string'},
            {'name': 'version', 'type': 'string'},
            {'name': 'chainId', 'type': 'uint256'},
            {'name': 'verifyingContract', 'type': 'string'},
            {'name': 'salt', 'type': 'string'},
        ],
        'Tx': [
            {'name': 'account_number', 'type': 'string'},
            {'name': 'chain_id', 'type': 'string'},
            {'name': 'fee', 'type': 'Fee'},
            {'name': 'memo', 'type': 'string'},
            {'name': 'msgs', 'type': 'Msg[]'},
            {'name': 'sequence', 'type': 'string'},
        ],
        'Fee': [
            {'name': 'feePayer', 'type': 'string'},
            {'name': 'amount', 'type': 'Coin[]'},
            {'name': 'gas', 'type': 'string'},
        ],
        'Coin': [
            {'name': 'denom', 'type': 'string'},
            {'name': 'amount', 'type': 'string'},
        ],
        'Msg': [
            {'name': 'type', 'type': 'string'},
            {'name': 'value', 'type': 'MsgValue'},
        ],
    }
    types.update(msg_values)
    return types


def generate_fee(amount: str, denom: str, gas: str, fee_payer: str) -> dict[str, Any]:
    """Generate fee definition structure."""
    return {
        'amount': [{'amount': amount, 'denom': denom}],
        'gas': gas,
        'feePayer': fee_payer,
    }



from __future__ import annotations

from typing import Any, Mapping

from betterproto import Message

from evmos.eip712 import (
    IBC_MSG_TRANSFER_TYPES,
    MSG_SEND_TYPES,
    MSG_VOTE_TYPES,
    create_ibc_msg_transfer,
    create_msg_send,
    create_msg_vote,
)
from evmos.proto import (
    AuthInfo,
    MessageGenerated,
    MsgSend,
    MsgTransfer,
    MsgVote,
    TxBody,
    TxRaw,
    create_any_message,
)
from evmos.proto import create_ibc_msg_transfer as proto_create_ibc_msg_transfer
from evmos.proto import create_msg_send as proto_msg_send
from evmos.proto import create_msg_vote as proto_create_msg_vote
from evmos.proto import create_tx_raw, create_web3_extension
from evmos.proto.autogen.py.ethermint.types.v1 import ExtensionOptionsWeb3Tx
from evmos.transactions.authz import (
    create_tx_msg_stake_authorization,
    create_tx_msg_stake_revoke_authorization,
)
from evmos.transactions.common import (
    Chain,
    Fee,
    Sender,
    TxGenerated,
    TxGeneratedBase,
    to_generated,
)
from evmos.transactions.erc20 import (
    create_tx_msg_convert_coin,
    create_tx_msg_convert_erc20,
)
from evmos.transactions.feesplit import (
    create_tx_msg_cancel_fee_split,
    create_tx_msg_register_fee_split,
    create_tx_msg_update_fee_split,
)
from evmos.transactions.staking import (
    create_tx_msg_begin_redelegate,
    create_tx_msg_delegate,
    create_tx_msg_multiple_withdraw_delegator_reward,
    create_tx_msg_undelegate,
    create_tx_msg_withdraw_delegator_reward,
    create_tx_msg_withdraw_validator_commission,
)
from evmos.transactions.validator import create_tx_msg_edit_validator

__all__ = [
    'create_tx_msg_vote',
    'create_tx_ibc_msg_transfer',
    'create_message_send',
    'create_tx_raw_eip712',
    'signature_to_web3_extension',
    'Chain',
    'Fee',
    'Sender',
    'TxGeneratedBase',
    'TxGenerated',
    'create_tx_msg_stake_authorization',
    'create_tx_msg_stake_revoke_authorization',
    'create_tx_msg_convert_coin',
    'create_tx_msg_convert_erc20',
    'create_tx_msg_cancel_fee_split',
    'create_tx_msg_register_fee_split',
    'create_tx_msg_update_fee_split',
    'create_tx_msg_delegate',
    'create_tx_msg_begin_redelegate',
    'create_tx_msg_undelegate',
    'create_tx_msg_withdraw_delegator_reward',
    'create_tx_msg_multiple_withdraw_delegator_reward',
    'create_tx_msg_withdraw_validator_commission',
    'create_tx_msg_edit_validator',
]


# gov.ts


@to_generated(MSG_VOTE_TYPES, proto=True)
def create_tx_msg_vote(
    account_address: str,
    proposal_id: int,
    option: int,
) -> tuple[Mapping[str, Any], MessageGenerated[MsgVote]]:
    """Create transaction with voting message."""
    # EIP712
    msg = create_msg_vote(
        proposal_id,
        option,
        account_address,
    )

    # Cosmos
    msg_cosmos = proto_create_msg_vote(
        proposal_id,
        option,
        account_address,
    )
    return msg, msg_cosmos


# ibcMessageTransfer.ts


@to_generated(IBC_MSG_TRANSFER_TYPES, proto=True)
def create_tx_ibc_msg_transfer(
    account_address: str,
    # Channel
    source_port: str,
    source_channel: str,
    # Token
    amount: str,
    denom: str,
    # Addresses
    receiver: str,
    # Timeout
    revision_number: int,
    revision_height: int,
    timeout_timestamp: str,
) -> tuple[Mapping[str, Any], MessageGenerated[MsgTransfer]]:
    """Create transaction with IBC transfer message."""
    # EIP712

    msg = create_ibc_msg_transfer(
        receiver,
        account_address,
        source_channel,
        source_port,
        revision_height,
        revision_number,
        timeout_timestamp,
        amount,
        denom,
    )

    # Cosmos
    msg_cosmos = proto_create_ibc_msg_transfer(
        source_port,
        source_channel,
        amount,
        denom,
        account_address,
        receiver,
        revision_number,
        revision_height,
        timeout_timestamp,
    )
    return msg, msg_cosmos


# msgSend.ts


@to_generated(MSG_SEND_TYPES, proto=True)
def create_message_send(
    account_address: str,
    destination_address: str,
    amount: str,
    denom: str,
) -> tuple[Mapping[str, Any], MessageGenerated[MsgSend]]:
    """Create transaction with message send."""
    # EIP712
    msg = create_msg_send(
        amount,
        denom,
        account_address,
        destination_address,
    )

    # Cosmos
    msg_send = proto_msg_send(
        account_address,
        destination_address,
        amount,
        denom,
    )
    return msg, msg_send


# txRaw.ts


def create_tx_raw_eip712(
    body: TxBody,
    auth_info: AuthInfo,
    extension: MessageGenerated[Message],
) -> MessageGenerated[TxRaw]:
    """Create a message with raw EIP712 transaction."""
    body.extension_options.append(create_any_message(extension))

    return create_tx_raw(bytes(body), bytes(auth_info), [b''])


# web3Extension.ts


def signature_to_web3_extension(
    chain: Chain,
    sender: Sender,
    signature: bytes,
) -> MessageGenerated[ExtensionOptionsWeb3Tx]:
    """Create a message with web3 extension from signature."""
    return create_web3_extension(
        chain.chain_id,
        sender.account_address,
        signature,
    )



from __future__ import annotations

from typing import Final

from evmos.eip712.base import MsgInterface

MSG_CONVERT_COIN_TYPES: Final = {
    'MsgValue': [
        {'name': 'coin', 'type': 'TypeCoin'},
        {'name': 'receiver', 'type': 'string'},
        {'name': 'sender', 'type': 'string'},
    ],
    'TypeCoin': [
        {'name': 'denom', 'type': 'string'},
        {'name': 'amount', 'type': 'string'},
    ],
}
"""Types for message for coin types conversion."""


def create_msg_convert_coin(
    denom: str | int,
    amount: str | int,
    receiver: str,
    sender: str,
) -> MsgInterface:
    """Create message for coin types conversion."""
    return {
        'type': 'evmos/MsgConvertCoin',
        'value': {
            'coin': {
                'denom': str(denom),
                'amount': str(amount),
            },
            'receiver': receiver,
            'sender': sender,
        },
    }


MSG_CONVERT_ERC20_TYPES: Final = {
    'MsgValue': [
        {'name': 'contract_address', 'type': 'string'},
        {'name': 'amount', 'type': 'string'},
        {'name': 'receiver', 'type': 'string'},
        {'name': 'sender', 'type': 'string'},
    ],
}
"""Types for message for ERC20 types conversion."""


def create_msg_convert_erc20(
    contract_address: str,
    amount: str,
    receiver: str,
    sender: str,
) -> MsgInterface:
    """Create message for ERC20 types conversion."""
    return {
        'type': 'evmos/MsgConvertERC20',
        'value': {
            'contract_address': contract_address,
            'amount': amount,
            'receiver': receiver,
            'sender': sender,
        },
    }



from __future__ import annotations

from evmos.proto.autogen.py.cosmos.base.v1beta1 import Coin
from evmos.proto.autogen.py.evmos.erc20.v1 import MsgConvertCoin, MsgConvertErc20
from evmos.proto.utils import MessageGenerated

# messageConvertCoin.ts


def create_msg_convert_coin(
    denom: str,
    amount: str,
    receiver: str,
    sender: str,
) -> MessageGenerated[MsgConvertCoin]:
    """Create message for coin conversion."""
    msg = MsgConvertCoin(
        coin=Coin(denom=denom, amount=amount),
        receiver=receiver,
        sender=sender,
    )
    return MessageGenerated(
        message=msg,
        path='evmos.erc20.v1.MsgConvertCoin',
    )


# messageConvertERC20.ts


def create_msg_convert_erc20(
    contract_address: str,
    amount: str,
    receiver: str,
    sender: str,
) -> MessageGenerated[MsgConvertErc20]:
    """Create message for ERC20 conversion."""
    msg = MsgConvertErc20(
        contract_address=contract_address,
        amount=amount,
        receiver=receiver,
        sender=sender,
    )
    return MessageGenerated(
        message=msg,
        path='evmos.erc20.v1.MsgConvertERC20',
    )



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: evmos/vesting/v1/query.proto, evmos/vesting/v1/tx.proto, evmos/vesting/v1/vesting.proto
# plugin: python-betterproto
# This file has been @generated
from dataclasses import dataclass
from datetime import datetime
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase

from ....cosmos.base import v1beta1 as ___cosmos_base_v1_beta1__
from ....cosmos.vesting import v1beta1 as ___cosmos_vesting_v1_beta1__


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class QueryBalancesRequest(betterproto.Message):
    """
    QueryBalancesRequest is the request type for the Query/Balances RPC method.
    """

    address: str = betterproto.string_field(1)
    """address of the clawback vesting account"""


@dataclass(eq=False, repr=False)
class QueryBalancesResponse(betterproto.Message):
    """
    QueryBalancesResponse is the response type for the Query/Balances RPC
    method.
    """

    locked: List['___cosmos_base_v1_beta1__.Coin'] = betterproto.message_field(1)
    """current amount of locked tokens"""

    unvested: List['___cosmos_base_v1_beta1__.Coin'] = betterproto.message_field(2)
    """current amount of unvested tokens"""

    vested: List['___cosmos_base_v1_beta1__.Coin'] = betterproto.message_field(3)
    """current amount of vested tokens"""


@dataclass(eq=False, repr=False)
class ClawbackVestingAccount(betterproto.Message):
    """
    ClawbackVestingAccount implements the VestingAccount interface. It provides
    an account that can hold contributions subject to "lockup" (like a
    PeriodicVestingAccount), or vesting which is subject to clawback
    of unvested tokens, or a combination (tokens vest, but are still locked).
    """

    base_vesting_account: '___cosmos_vesting_v1_beta1__.BaseVestingAccount' = (
        betterproto.message_field(1)
    )
    """
    base_vesting_account implements the VestingAccount interface. It contains
    all the necessary fields needed for any vesting account implementation
    """

    funder_address: str = betterproto.string_field(2)
    """funder_address specifies the account which can perform clawback"""

    start_time: datetime = betterproto.message_field(3)
    """start_time defines the time at which the vesting period begins"""

    lockup_periods: List[
        '___cosmos_vesting_v1_beta1__.Period'
    ] = betterproto.message_field(4)
    """
    lockup_periods defines the unlocking schedule relative to the start_time
    """

    vesting_periods: List[
        '___cosmos_vesting_v1_beta1__.Period'
    ] = betterproto.message_field(5)
    """
    vesting_periods defines the vesting schedule relative to the start_time
    """


@dataclass(eq=False, repr=False)
class MsgCreateClawbackVestingAccount(betterproto.Message):
    """
    MsgCreateClawbackVestingAccount defines a message that enables creating a
    ClawbackVestingAccount.
    """

    from_address: str = betterproto.string_field(1)
    """
    from_address specifies the account to provide the funds and sign the
    clawback request
    """

    to_address: str = betterproto.string_field(2)
    """to_address specifies the account to receive the funds"""

    start_time: datetime = betterproto.message_field(3)
    """start_time defines the time at which the vesting period begins"""

    lockup_periods: List[
        '___cosmos_vesting_v1_beta1__.Period'
    ] = betterproto.message_field(4)
    """
    lockup_periods defines the unlocking schedule relative to the start_time
    """

    vesting_periods: List[
        '___cosmos_vesting_v1_beta1__.Period'
    ] = betterproto.message_field(5)
    """
    vesting_periods defines thevesting schedule relative to the start_time
    """

    merge: bool = betterproto.bool_field(6)
    """
    merge specifies a the creation mechanism for existing
    ClawbackVestingAccounts. If true, merge this new grant into an existing
    ClawbackVestingAccount, or create it if it does not exist. If false,
    creates a new account. New grants to an existing account must be from the
    same from_address.
    """


@dataclass(eq=False, repr=False)
class MsgCreateClawbackVestingAccountResponse(betterproto.Message):
    """
    MsgCreateClawbackVestingAccountResponse defines the
    MsgCreateClawbackVestingAccount response type.
    """

    pass


@dataclass(eq=False, repr=False)
class MsgClawback(betterproto.Message):
    """
    MsgClawback defines a message that removes unvested tokens from a
    ClawbackVestingAccount.
    """

    funder_address: str = betterproto.string_field(1)
    """funder_address is the address which funded the account"""

    account_address: str = betterproto.string_field(2)
    """
    account_address is the address of the ClawbackVestingAccount to claw back
    from.
    """

    dest_address: str = betterproto.string_field(3)
    """
    dest_address specifies where the clawed-back tokens should be transferred
    to. If empty, the tokens will be transferred back to the original funder of
    the account.
    """


@dataclass(eq=False, repr=False)
class MsgClawbackResponse(betterproto.Message):
    """MsgClawbackResponse defines the MsgClawback response type."""

    pass


class QueryStub(betterproto.ServiceStub):
    async def balances(
        self,
        query_balances_request: 'QueryBalancesRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryBalancesResponse':
        return await self._unary_unary(
            '/evmos.vesting.v1.Query/Balances',
            query_balances_request,
            QueryBalancesResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class MsgStub(betterproto.ServiceStub):
    async def create_clawback_vesting_account(
        self,
        msg_create_clawback_vesting_account: 'MsgCreateClawbackVestingAccount',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'MsgCreateClawbackVestingAccountResponse':
        return await self._unary_unary(
            '/evmos.vesting.v1.Msg/CreateClawbackVestingAccount',
            msg_create_clawback_vesting_account,
            MsgCreateClawbackVestingAccountResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def clawback(
        self,
        msg_clawback: 'MsgClawback',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'MsgClawbackResponse':
        return await self._unary_unary(
            '/evmos.vesting.v1.Msg/Clawback',
            msg_clawback,
            MsgClawbackResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class QueryBase(ServiceBase):
    async def balances(
        self, query_balances_request: 'QueryBalancesRequest'
    ) -> 'QueryBalancesResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_balances(
        self,
        stream: 'grpclib.server.Stream[QueryBalancesRequest, QueryBalancesResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.balances(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            '/evmos.vesting.v1.Query/Balances': grpclib.const.Handler(
                self.__rpc_balances,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryBalancesRequest,
                QueryBalancesResponse,
            ),
        }


class MsgBase(ServiceBase):
    async def create_clawback_vesting_account(
        self, msg_create_clawback_vesting_account: 'MsgCreateClawbackVestingAccount'
    ) -> 'MsgCreateClawbackVestingAccountResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def clawback(self, msg_clawback: 'MsgClawback') -> 'MsgClawbackResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_create_clawback_vesting_account(
        self,
        stream: 'grpclib.server.Stream[MsgCreateClawbackVestingAccount, MsgCreateClawbackVestingAccountResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.create_clawback_vesting_account(request)
        await stream.send_message(response)

    async def __rpc_clawback(
        self, stream: 'grpclib.server.Stream[MsgClawback, MsgClawbackResponse]'
    ) -> None:
        request = await stream.recv_message()
        response = await self.clawback(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            '/evmos.vesting.v1.Msg/CreateClawbackVestingAccount': grpclib.const.Handler(
                self.__rpc_create_clawback_vesting_account,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgCreateClawbackVestingAccount,
                MsgCreateClawbackVestingAccountResponse,
            ),
            '/evmos.vesting.v1.Msg/Clawback': grpclib.const.Handler(
                self.__rpc_clawback,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgClawback,
                MsgClawbackResponse,
            ),
        }



from __future__ import annotations

from enum import Enum
from typing import Sequence, TypedDict

from typing_extensions import NotRequired

# Enums


class BroadcastMode(str, Enum):
    """Broadcasting mode."""

    UNSPECIFIED = 'BROADCAST_MODE_UNSPECIFIED'
    """Broadcasting mode unset."""
    BLOCK = 'BROADCAST_MODE_BLOCK'
    """Broadcasting mode - block."""
    SYNC = 'BROADCAST_MODE_SYNC'
    """Broadcasting mode - synchronous."""
    ASYNC = 'BROADCAST_MODE_ASYNC'
    """Broadcasting mode - asynchronous."""


class ProposalStatus(str, Enum):
    """Proposal status."""

    UNSPECIFIED = 'PROPOSAL_STATUS_UNSPECIFIED'
    """Proposal status unset."""
    DEPOSIT = 'PROPOSAL_STATUS_DEPOSIT_PERIOD'
    """Proposal during deposit period."""
    VOTING = 'PROPOSAL_STATUS_VOTING_PERIOD'
    """Proposal during voting period."""
    PASSED = 'PROPOSAL_STATUS_PASSED'
    """Proposal passed."""
    REJECTED = 'PROPOSAL_STATUS_REJECTED'
    """Proposal rejected."""
    FAILED = 'PROPOSAL_STATUS_FAILED'
    """Proposal failed."""


# Functions


def generate_endpoint_account(address: str) -> str:
    """Generate endpoint for account details."""
    return f'/cosmos/auth/v1beta1/accounts/{address}'


def generate_endpoint_balances(address: str) -> str:
    """Generate endpoint for account balances."""
    return f'/cosmos/bank/v1beta1/balances/{address}'


def generate_endpoint_broadcast() -> str:
    """Generate endpoint for broadcasting."""
    return '/cosmos/tx/v1beta1/txs'


def generate_endpoint_proposals() -> str:
    """Generate proposals endpoint.

    Note:
      This returns all the proposals.
    """
    # TODO: add pagination to the request
    return '/cosmos/gov/v1beta1/proposals'


def generate_endpoint_claims_record(address: str) -> str:
    """Generate endpoint for record claims."""
    return f'/evmos/claims/v1/claims_records/{address}'


def generate_endpoint_proposal_tally(proposal_id: str) -> str:
    """Generate endpoint for tally proposals."""
    return f'/cosmos/gov/v1beta1/proposals/{proposal_id}/tally'


def generate_endpoint_ibc_channels() -> str:
    """Get all the IBC channels."""
    return '/ibc/core/channel/v1/channels'


def generate_endpoint_distribution_rewards_by_address(address: str) -> str:
    """Generate endpoint for rewards distribution."""
    return f'/cosmos/distribution/v1beta1/delegators/${address}/rewards'


def generate_endpoint_get_validators() -> str:
    """Generate endpoint for validators list."""
    return '/cosmos/staking/v1beta1/validators'


def generate_endpoint_get_delegations(delegator_address: str) -> str:
    """Generate endpoint for delegation."""
    return f'/cosmos/staking/v1beta1/delegations/${delegator_address}'


def generate_endpoint_get_undelegations(delegator_address: str) -> str:
    """Generate endpoint for undelegation."""
    return (
        f'/cosmos/staking/v1beta1/delegators/${delegator_address}/unbonding_delegations'
    )


# account.ts


class PubkeyAccountResponse(TypedDict):
    """Response with account public key.

    Also has "@type" key with corresponding `str` value.
    """

    key: str
    """Account public key (base64 encoded)."""


class BaseAccountResponse(TypedDict):
    """``base_account`` response field.

    This is similar to :class:`evmos.transactions.common.Sender`.
    """

    address: str
    """Account address (bech32)."""
    pub_key: NotRequired[PubkeyAccountResponse]
    """Account public key as dict."""
    account_number: str
    """Account internal number."""
    sequence: str
    """Account nonce - number of previously sent transactions."""


class AccountResponseBody(TypedDict):
    """Account body from response.

    Also has "@type" key with corresponding `str` value.
    """

    base_account: BaseAccountResponse
    """Account response content."""
    code_hash: str
    """Response status."""


class AccountResponse(TypedDict):
    """Full response of account endpoint."""

    account: AccountResponseBody
    """Account full response content, wrapped in another dict."""


# balance.ts


class PaginationResponse(TypedDict):
    """Pagination response part."""

    next_key: str
    """Next key to fetch."""
    total: int
    """Total amount of records."""


class BalancesResponse(TypedDict):
    """Response of balance endpoint."""

    balances: Sequence[Coin]
    """All coin balances of account."""
    pagination: PaginationResponse
    """Pagination part."""


# broadcast.ts


class BroadcastPostBody(TypedDict):
    """Body of POST request for transaction broadcasting."""

    tx_bytes: str
    """base64-encoded transaction bytes."""
    mode: str
    """Broadcasting mode."""


# claims.ts


class Claim(TypedDict):
    """Single claim record."""

    action: str
    """Claiming action."""
    completed: bool
    """Whether claim is completed."""
    claimable_amount: str
    """Amount to claim."""


class ClaimsRecordResponse(TypedDict):
    """Response of claims endpoint."""

    initial_claimable_amount: int
    """Amount available to claim before processing."""
    claims: Sequence[Claim]
    """All claims performed."""


# coin.ts


class Coin(TypedDict):
    """Coin structure."""

    denom: str
    """Coin denomination."""
    amount: str
    """Amount, as string (like '1000')."""


# gov.ts


class ProposalContent(TypedDict):
    """Proposal content.

    Also has "@type" key with corresponding `str` value.
    """

    title: str
    """Proposal title."""
    description: str
    """Proposal description."""


class TallyResults(TypedDict):
    """Tally results for a single proposal."""

    yes: str
    """Number of positive votes, as string (like '1234')."""
    abstain: str
    """Number of abstain votes, as string (like '1234')."""
    no: str
    """Number of negative votes, as string (like '1234')."""
    no_with_veto: str
    """Number of negative votes with veto, as string (like '1234')."""


class Proposal(TypedDict):
    """Single proposal."""

    proposal_id: str
    """Internal proposal ID."""
    content: ProposalContent
    """Proposal content."""
    status: str
    """Proposal status."""
    final_tally_result: TallyResults
    """Proposal final voting results."""
    submit_time: str
    """Proposal submission time, as timestamp str."""
    deposit_end_time: str
    """Proposal deposit end time, as timestamp str."""
    total_deposit: Sequence[Coin]
    """Proposal total deposit, as str number."""
    voting_start_time: str
    """Proposal voting start time, as timestamp str."""
    voting_end_time: str
    """Proposal voting end time, as timestamp str."""


class ProposalsResponse(TypedDict):
    """Response type of proposals endpoint."""

    proposals: Sequence[Proposal]
    """Response of proposals endpoint."""
    pagination: PaginationResponse
    """Pagination block."""


class TallyResponse(TypedDict):
    """Response type of tally endpoint."""

    tally: TallyResults
    """Results."""


# ibc.ts


class CounterParty(TypedDict):
    """IBC counterparty."""

    port_id: str
    """IBC port ID."""
    channel_id: str
    """IBC channel ID."""


class IBCChannel(TypedDict):
    """IBC channel."""

    state: str
    """IBC channel state."""
    ordering: str
    """IBC channel ordering."""
    counterparty: CounterParty
    """IBC channel counterparty."""
    connection_hops: Sequence[str]
    """IBC channel connection hops."""
    version: str
    """IBC channel version."""
    port_id: str
    """IBC channel port id."""
    channel_id: str
    """IBC channel id."""


class ChannelsResponseHeight(TypedDict):
    """``height`` field type of channels endpoint response."""

    revision_number: str
    """Current revision number."""
    revision_height: str
    """Current revision height."""


class ChannelsResponse(TypedDict):
    """Response type of channels endpoint."""

    channels: Sequence[IBCChannel]
    """Actual channels."""
    pagination: PaginationResponse
    """Pagination block."""
    height: ChannelsResponseHeight
    """Response height."""


# staking.ts


class Reward(TypedDict):
    """Reward."""

    validator_address: str
    """Validator address."""
    reward: Sequence[Coin]
    """Reward (in all coin types)."""


class DistributionRewardsResponse(TypedDict):
    """Response type of rewards distribution endpoint."""

    rewards: Sequence[Reward]
    """Reward (in all coin types)."""
    total: Sequence[Coin]
    """Total rewards (in all coin types)."""


class ValidatorCommissionRates(TypedDict):
    """Validator commission rates."""

    max_change_rate: str
    """Max change rate, number as str."""
    max_rate: str
    """Max overall rate, number as str."""
    rate: str
    """Rate, number as str."""


class ValidatorCommission(TypedDict):
    """Commission rates of validator."""

    commission_rates: ValidatorCommissionRates
    """Actual rates."""
    update_time: str
    """Last update time, as str timestamp."""


class ValidatorDescription(TypedDict):
    """Validator details."""

    details: str
    """Validator details."""
    identity: str
    """Validator identity."""
    moniker: str
    """Validator moniker."""
    security_contact: str
    """Validator security contact."""
    website: str
    """Validator website."""


class Validator(TypedDict):
    """Validator definition."""

    commission: ValidatorCommission
    """Validator commissions."""
    consensus_pubkey: PubkeyAccountResponse
    """Consensus public key."""
    delegator_shares: str
    """Total delegator shares."""
    description: ValidatorDescription
    """Description."""
    jailed: bool
    """Whether validator is jailed."""
    min_self_delegation: str
    """Minimal self delegation amount, int as str."""
    operator_address: str
    """Operator address."""
    status: str
    """Validator status."""
    tokens: str
    """Validator tokens."""
    unbonding_height: str
    """Validator unbounding height."""
    unbonding_time: str
    """Validator unbounding time."""


class GetValidatorsResponse(TypedDict):
    """Response type of validators endpoint."""

    validators: Sequence[Validator]
    """All validators present."""
    pagination: PaginationResponse
    """Pagination block."""


class DelegationParams(TypedDict):
    """Parameters of delegation deal."""

    delegator_address: str
    """Address of delegator."""
    shares: str
    """Total delegator shares."""
    validator_address: str
    """Validator address."""


class DelegationResponse(TypedDict):
    """Single delegation item received from delegation endpoint."""

    balance: Coin
    """Balance of delegated coin."""
    delegation: DelegationParams
    """Deal options."""


class GetDelegationsResponse(TypedDict):
    """Response type of delegation endpoint."""

    delegation_responses: Sequence[DelegationResponse]
    """All response blocks."""
    pagination: PaginationResponse
    """Pagination block."""


class UndelegationEntry(TypedDict):
    """Undelegation entry."""

    creation_height: str
    """Height of block when this item was created."""
    completion_time: str
    """Completion time."""
    initial_balance: str
    """Initial owner balance."""
    balance: str
    """Resulting owner balance."""


class UndelegationResponse(TypedDict):
    """Single undelegation item received from delegation endpoint."""

    delegator_address: str
    """Delegator address."""
    validator_address: str
    """Validator address."""
    entries: Sequence[UndelegationEntry]
    """Actual undelegation descriptions."""


class GetUndelegationsResponse(TypedDict):
    """Response type of undelegation endpoint."""

    unbonding_responses: Sequence[UndelegationResponse]
    """All responses as a sequence."""
    pagination: PaginationResponse
    """Pagination block."""



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: ibc/core/connection/v1/connection.proto, ibc/core/connection/v1/genesis.proto, ibc/core/connection/v1/query.proto, ibc/core/connection/v1/tx.proto
# plugin: python-betterproto
# This file has been @generated
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase

from .....cosmos.base.query import v1beta1 as ____cosmos_base_query_v1_beta1__
from ...client import v1 as __client_v1__
from ...commitment import v1 as __commitment_v1__


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


class State(betterproto.Enum):
    """
    State defines if a connection is in one of the following states:
    INIT, TRYOPEN, OPEN or UNINITIALIZED.
    """

    STATE_UNINITIALIZED_UNSPECIFIED = 0
    """Default State"""

    STATE_INIT = 1
    """A connection end has just started the opening handshake."""

    STATE_TRYOPEN = 2
    """
    A connection end has acknowledged the handshake step on the counterparty
    chain.
    """

    STATE_OPEN = 3
    """A connection end has completed the handshake."""


@dataclass(eq=False, repr=False)
class ConnectionEnd(betterproto.Message):
    """
    ConnectionEnd defines a stateful object on a chain connected to another
    separate one.
    NOTE: there must only be 2 defined ConnectionEnds to establish
    a connection between two chains.
    """

    client_id: str = betterproto.string_field(1)
    """client associated with this connection."""

    versions: List['Version'] = betterproto.message_field(2)
    """
    IBC version which can be utilised to determine encodings or protocols for
    channels or packets utilising this connection.
    """

    state: 'State' = betterproto.enum_field(3)
    """current state of the connection end."""

    counterparty: 'Counterparty' = betterproto.message_field(4)
    """counterparty chain associated with this connection."""

    delay_period: int = betterproto.uint64_field(5)
    """
    delay period that must pass before a consensus state can be used for
    packet-verification NOTE: delay period logic is only implemented by some
    clients.
    """


@dataclass(eq=False, repr=False)
class IdentifiedConnection(betterproto.Message):
    """
    IdentifiedConnection defines a connection with additional connection
    identifier field.
    """

    id: str = betterproto.string_field(1)
    """connection identifier."""

    client_id: str = betterproto.string_field(2)
    """client associated with this connection."""

    versions: List['Version'] = betterproto.message_field(3)
    """
    IBC version which can be utilised to determine encodings or protocols for
    channels or packets utilising this connection
    """

    state: 'State' = betterproto.enum_field(4)
    """current state of the connection end."""

    counterparty: 'Counterparty' = betterproto.message_field(5)
    """counterparty chain associated with this connection."""

    delay_period: int = betterproto.uint64_field(6)
    """delay period associated with this connection."""


@dataclass(eq=False, repr=False)
class Counterparty(betterproto.Message):
    """
    Counterparty defines the counterparty chain associated with a connection end.
    """

    client_id: str = betterproto.string_field(1)
    """
    identifies the client on the counterparty chain associated with a given
    connection.
    """

    connection_id: str = betterproto.string_field(2)
    """
    identifies the connection end on the counterparty chain associated with a
    given connection.
    """

    prefix: '__commitment_v1__.MerklePrefix' = betterproto.message_field(3)
    """commitment merkle prefix of the counterparty chain."""


@dataclass(eq=False, repr=False)
class ClientPaths(betterproto.Message):
    """ClientPaths define all the connection paths for a client state."""

    paths: List[str] = betterproto.string_field(1)
    """list of connection paths"""


@dataclass(eq=False, repr=False)
class ConnectionPaths(betterproto.Message):
    """
    ConnectionPaths define all the connection paths for a given client state.
    """

    client_id: str = betterproto.string_field(1)
    """client state unique identifier"""

    paths: List[str] = betterproto.string_field(2)
    """list of connection paths"""


@dataclass(eq=False, repr=False)
class Version(betterproto.Message):
    """
    Version defines the versioning scheme used to negotiate the IBC verison in
    the connection handshake.
    """

    identifier: str = betterproto.string_field(1)
    """unique version identifier"""

    features: List[str] = betterproto.string_field(2)
    """list of features compatible with the specified identifier"""


@dataclass(eq=False, repr=False)
class Params(betterproto.Message):
    """Params defines the set of Connection parameters."""

    max_expected_time_per_block: int = betterproto.uint64_field(1)
    """
    maximum expected time per block (in nanoseconds), used to enforce block
    delay. This parameter should reflect the largest amount of time that the
    chain might reasonably take to produce the next block under normal
    operating conditions. A safe choice is 3-5x the expected time per block.
    """


@dataclass(eq=False, repr=False)
class QueryConnectionRequest(betterproto.Message):
    """
    QueryConnectionRequest is the request type for the Query/Connection RPC
    method
    """

    connection_id: str = betterproto.string_field(1)
    """connection unique identifier"""


@dataclass(eq=False, repr=False)
class QueryConnectionResponse(betterproto.Message):
    """
    QueryConnectionResponse is the response type for the Query/Connection RPC
    method. Besides the connection end, it includes a proof and the height from
    which the proof was retrieved.
    """

    connection: 'ConnectionEnd' = betterproto.message_field(1)
    """connection associated with the request identifier"""

    proof: bytes = betterproto.bytes_field(2)
    """merkle proof of existence"""

    proof_height: '__client_v1__.Height' = betterproto.message_field(3)
    """height at which the proof was retrieved"""


@dataclass(eq=False, repr=False)
class QueryConnectionsRequest(betterproto.Message):
    """
    QueryConnectionsRequest is the request type for the Query/Connections RPC
    method
    """

    pagination: '____cosmos_base_query_v1_beta1__.PageRequest' = (
        betterproto.message_field(1)
    )


@dataclass(eq=False, repr=False)
class QueryConnectionsResponse(betterproto.Message):
    """
    QueryConnectionsResponse is the response type for the Query/Connections RPC
    method.
    """

    connections: List['IdentifiedConnection'] = betterproto.message_field(1)
    """list of stored connections of the chain."""

    pagination: '____cosmos_base_query_v1_beta1__.PageResponse' = (
        betterproto.message_field(2)
    )
    """pagination response"""

    height: '__client_v1__.Height' = betterproto.message_field(3)
    """query block height"""


@dataclass(eq=False, repr=False)
class QueryClientConnectionsRequest(betterproto.Message):
    """
    QueryClientConnectionsRequest is the request type for the
    Query/ClientConnections RPC method
    """

    client_id: str = betterproto.string_field(1)
    """client identifier associated with a connection"""


@dataclass(eq=False, repr=False)
class QueryClientConnectionsResponse(betterproto.Message):
    """
    QueryClientConnectionsResponse is the response type for the
    Query/ClientConnections RPC method
    """

    connection_paths: List[str] = betterproto.string_field(1)
    """slice of all the connection paths associated with a client."""

    proof: bytes = betterproto.bytes_field(2)
    """merkle proof of existence"""

    proof_height: '__client_v1__.Height' = betterproto.message_field(3)
    """height at which the proof was generated"""


@dataclass(eq=False, repr=False)
class QueryConnectionClientStateRequest(betterproto.Message):
    """
    QueryConnectionClientStateRequest is the request type for the
    Query/ConnectionClientState RPC method
    """

    connection_id: str = betterproto.string_field(1)
    """connection identifier"""


@dataclass(eq=False, repr=False)
class QueryConnectionClientStateResponse(betterproto.Message):
    """
    QueryConnectionClientStateResponse is the response type for the
    Query/ConnectionClientState RPC method
    """

    identified_client_state: '__client_v1__.IdentifiedClientState' = (
        betterproto.message_field(1)
    )
    """client state associated with the channel"""

    proof: bytes = betterproto.bytes_field(2)
    """merkle proof of existence"""

    proof_height: '__client_v1__.Height' = betterproto.message_field(3)
    """height at which the proof was retrieved"""


@dataclass(eq=False, repr=False)
class QueryConnectionConsensusStateRequest(betterproto.Message):
    """
    QueryConnectionConsensusStateRequest is the request type for the
    Query/ConnectionConsensusState RPC method
    """

    connection_id: str = betterproto.string_field(1)
    """connection identifier"""

    revision_number: int = betterproto.uint64_field(2)
    revision_height: int = betterproto.uint64_field(3)


@dataclass(eq=False, repr=False)
class QueryConnectionConsensusStateResponse(betterproto.Message):
    """
    QueryConnectionConsensusStateResponse is the response type for the
    Query/ConnectionConsensusState RPC method
    """

    consensus_state: 'betterproto_lib_google_protobuf.Any' = betterproto.message_field(
        1
    )
    """consensus state associated with the channel"""

    client_id: str = betterproto.string_field(2)
    """client ID associated with the consensus state"""

    proof: bytes = betterproto.bytes_field(3)
    """merkle proof of existence"""

    proof_height: '__client_v1__.Height' = betterproto.message_field(4)
    """height at which the proof was retrieved"""


@dataclass(eq=False, repr=False)
class GenesisState(betterproto.Message):
    """GenesisState defines the ibc connection submodule's genesis state."""

    connections: List['IdentifiedConnection'] = betterproto.message_field(1)
    client_connection_paths: List['ConnectionPaths'] = betterproto.message_field(2)
    next_connection_sequence: int = betterproto.uint64_field(3)
    """the sequence for the next generated connection identifier"""

    params: 'Params' = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class MsgConnectionOpenInit(betterproto.Message):
    """
    MsgConnectionOpenInit defines the msg sent by an account on Chain A to
    initialize a connection with Chain B.
    """

    client_id: str = betterproto.string_field(1)
    counterparty: 'Counterparty' = betterproto.message_field(2)
    version: 'Version' = betterproto.message_field(3)
    delay_period: int = betterproto.uint64_field(4)
    signer: str = betterproto.string_field(5)


@dataclass(eq=False, repr=False)
class MsgConnectionOpenInitResponse(betterproto.Message):
    """
    MsgConnectionOpenInitResponse defines the Msg/ConnectionOpenInit response
    type.
    """

    pass


@dataclass(eq=False, repr=False)
class MsgConnectionOpenTry(betterproto.Message):
    """
    MsgConnectionOpenTry defines a msg sent by a Relayer to try to open a
    connection on Chain B.
    """

    client_id: str = betterproto.string_field(1)
    previous_connection_id: str = betterproto.string_field(2)
    """
    in the case of crossing hello's, when both chains call OpenInit, we need
    the connection identifier of the previous connection in state INIT
    """

    client_state: 'betterproto_lib_google_protobuf.Any' = betterproto.message_field(3)
    counterparty: 'Counterparty' = betterproto.message_field(4)
    delay_period: int = betterproto.uint64_field(5)
    counterparty_versions: List['Version'] = betterproto.message_field(6)
    proof_height: '__client_v1__.Height' = betterproto.message_field(7)
    proof_init: bytes = betterproto.bytes_field(8)
    """
    proof of the initialization the connection on Chain A: `UNITIALIZED ->
    INIT`
    """

    proof_client: bytes = betterproto.bytes_field(9)
    """proof of client state included in message"""

    proof_consensus: bytes = betterproto.bytes_field(10)
    """proof of client consensus state"""

    consensus_height: '__client_v1__.Height' = betterproto.message_field(11)
    signer: str = betterproto.string_field(12)


@dataclass(eq=False, repr=False)
class MsgConnectionOpenTryResponse(betterproto.Message):
    """
    MsgConnectionOpenTryResponse defines the Msg/ConnectionOpenTry response type.
    """

    pass


@dataclass(eq=False, repr=False)
class MsgConnectionOpenAck(betterproto.Message):
    """
    MsgConnectionOpenAck defines a msg sent by a Relayer to Chain A to
    acknowledge the change of connection state to TRYOPEN on Chain B.
    """

    connection_id: str = betterproto.string_field(1)
    counterparty_connection_id: str = betterproto.string_field(2)
    version: 'Version' = betterproto.message_field(3)
    client_state: 'betterproto_lib_google_protobuf.Any' = betterproto.message_field(4)
    proof_height: '__client_v1__.Height' = betterproto.message_field(5)
    proof_try: bytes = betterproto.bytes_field(6)
    """
    proof of the initialization the connection on Chain B: `UNITIALIZED ->
    TRYOPEN`
    """

    proof_client: bytes = betterproto.bytes_field(7)
    """proof of client state included in message"""

    proof_consensus: bytes = betterproto.bytes_field(8)
    """proof of client consensus state"""

    consensus_height: '__client_v1__.Height' = betterproto.message_field(9)
    signer: str = betterproto.string_field(10)


@dataclass(eq=False, repr=False)
class MsgConnectionOpenAckResponse(betterproto.Message):
    """
    MsgConnectionOpenAckResponse defines the Msg/ConnectionOpenAck response type.
    """

    pass


@dataclass(eq=False, repr=False)
class MsgConnectionOpenConfirm(betterproto.Message):
    """
    MsgConnectionOpenConfirm defines a msg sent by a Relayer to Chain B to
    acknowledge the change of connection state to OPEN on Chain A.
    """

    connection_id: str = betterproto.string_field(1)
    proof_ack: bytes = betterproto.bytes_field(2)
    """
    proof for the change of the connection state on Chain A: `INIT -> OPEN`
    """

    proof_height: '__client_v1__.Height' = betterproto.message_field(3)
    signer: str = betterproto.string_field(4)


@dataclass(eq=False, repr=False)
class MsgConnectionOpenConfirmResponse(betterproto.Message):
    """
    MsgConnectionOpenConfirmResponse defines the Msg/ConnectionOpenConfirm
    response type.
    """

    pass


class QueryStub(betterproto.ServiceStub):
    async def connection(
        self,
        query_connection_request: 'QueryConnectionRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryConnectionResponse':
        return await self._unary_unary(
            '/ibc.core.connection.v1.Query/Connection',
            query_connection_request,
            QueryConnectionResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def connections(
        self,
        query_connections_request: 'QueryConnectionsRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryConnectionsResponse':
        return await self._unary_unary(
            '/ibc.core.connection.v1.Query/Connections',
            query_connections_request,
            QueryConnectionsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def client_connections(
        self,
        query_client_connections_request: 'QueryClientConnectionsRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryClientConnectionsResponse':
        return await self._unary_unary(
            '/ibc.core.connection.v1.Query/ClientConnections',
            query_client_connections_request,
            QueryClientConnectionsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def connection_client_state(
        self,
        query_connection_client_state_request: 'QueryConnectionClientStateRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryConnectionClientStateResponse':
        return await self._unary_unary(
            '/ibc.core.connection.v1.Query/ConnectionClientState',
            query_connection_client_state_request,
            QueryConnectionClientStateResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def connection_consensus_state(
        self,
        query_connection_consensus_state_request: 'QueryConnectionConsensusStateRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryConnectionConsensusStateResponse':
        return await self._unary_unary(
            '/ibc.core.connection.v1.Query/ConnectionConsensusState',
            query_connection_consensus_state_request,
            QueryConnectionConsensusStateResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class MsgStub(betterproto.ServiceStub):
    async def connection_open_init(
        self,
        msg_connection_open_init: 'MsgConnectionOpenInit',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'MsgConnectionOpenInitResponse':
        return await self._unary_unary(
            '/ibc.core.connection.v1.Msg/ConnectionOpenInit',
            msg_connection_open_init,
            MsgConnectionOpenInitResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def connection_open_try(
        self,
        msg_connection_open_try: 'MsgConnectionOpenTry',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'MsgConnectionOpenTryResponse':
        return await self._unary_unary(
            '/ibc.core.connection.v1.Msg/ConnectionOpenTry',
            msg_connection_open_try,
            MsgConnectionOpenTryResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def connection_open_ack(
        self,
        msg_connection_open_ack: 'MsgConnectionOpenAck',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'MsgConnectionOpenAckResponse':
        return await self._unary_unary(
            '/ibc.core.connection.v1.Msg/ConnectionOpenAck',
            msg_connection_open_ack,
            MsgConnectionOpenAckResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def connection_open_confirm(
        self,
        msg_connection_open_confirm: 'MsgConnectionOpenConfirm',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'MsgConnectionOpenConfirmResponse':
        return await self._unary_unary(
            '/ibc.core.connection.v1.Msg/ConnectionOpenConfirm',
            msg_connection_open_confirm,
            MsgConnectionOpenConfirmResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class QueryBase(ServiceBase):
    async def connection(
        self, query_connection_request: 'QueryConnectionRequest'
    ) -> 'QueryConnectionResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def connections(
        self, query_connections_request: 'QueryConnectionsRequest'
    ) -> 'QueryConnectionsResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def client_connections(
        self, query_client_connections_request: 'QueryClientConnectionsRequest'
    ) -> 'QueryClientConnectionsResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def connection_client_state(
        self, query_connection_client_state_request: 'QueryConnectionClientStateRequest'
    ) -> 'QueryConnectionClientStateResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def connection_consensus_state(
        self,
        query_connection_consensus_state_request: 'QueryConnectionConsensusStateRequest',
    ) -> 'QueryConnectionConsensusStateResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_connection(
        self,
        stream: 'grpclib.server.Stream[QueryConnectionRequest, QueryConnectionResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.connection(request)
        await stream.send_message(response)

    async def __rpc_connections(
        self,
        stream: 'grpclib.server.Stream[QueryConnectionsRequest, QueryConnectionsResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.connections(request)
        await stream.send_message(response)

    async def __rpc_client_connections(
        self,
        stream: 'grpclib.server.Stream[QueryClientConnectionsRequest, QueryClientConnectionsResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.client_connections(request)
        await stream.send_message(response)

    async def __rpc_connection_client_state(
        self,
        stream: 'grpclib.server.Stream[QueryConnectionClientStateRequest, QueryConnectionClientStateResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.connection_client_state(request)
        await stream.send_message(response)

    async def __rpc_connection_consensus_state(
        self,
        stream: 'grpclib.server.Stream[QueryConnectionConsensusStateRequest, QueryConnectionConsensusStateResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.connection_consensus_state(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            '/ibc.core.connection.v1.Query/Connection': grpclib.const.Handler(
                self.__rpc_connection,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryConnectionRequest,
                QueryConnectionResponse,
            ),
            '/ibc.core.connection.v1.Query/Connections': grpclib.const.Handler(
                self.__rpc_connections,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryConnectionsRequest,
                QueryConnectionsResponse,
            ),
            '/ibc.core.connection.v1.Query/ClientConnections': grpclib.const.Handler(
                self.__rpc_client_connections,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryClientConnectionsRequest,
                QueryClientConnectionsResponse,
            ),
            '/ibc.core.connection.v1.Query/ConnectionClientState': grpclib.const.Handler(
                self.__rpc_connection_client_state,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryConnectionClientStateRequest,
                QueryConnectionClientStateResponse,
            ),
            '/ibc.core.connection.v1.Query/ConnectionConsensusState': grpclib.const.Handler(
                self.__rpc_connection_consensus_state,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryConnectionConsensusStateRequest,
                QueryConnectionConsensusStateResponse,
            ),
        }


class MsgBase(ServiceBase):
    async def connection_open_init(
        self, msg_connection_open_init: 'MsgConnectionOpenInit'
    ) -> 'MsgConnectionOpenInitResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def connection_open_try(
        self, msg_connection_open_try: 'MsgConnectionOpenTry'
    ) -> 'MsgConnectionOpenTryResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def connection_open_ack(
        self, msg_connection_open_ack: 'MsgConnectionOpenAck'
    ) -> 'MsgConnectionOpenAckResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def connection_open_confirm(
        self, msg_connection_open_confirm: 'MsgConnectionOpenConfirm'
    ) -> 'MsgConnectionOpenConfirmResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_connection_open_init(
        self,
        stream: 'grpclib.server.Stream[MsgConnectionOpenInit, MsgConnectionOpenInitResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.connection_open_init(request)
        await stream.send_message(response)

    async def __rpc_connection_open_try(
        self,
        stream: 'grpclib.server.Stream[MsgConnectionOpenTry, MsgConnectionOpenTryResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.connection_open_try(request)
        await stream.send_message(response)

    async def __rpc_connection_open_ack(
        self,
        stream: 'grpclib.server.Stream[MsgConnectionOpenAck, MsgConnectionOpenAckResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.connection_open_ack(request)
        await stream.send_message(response)

    async def __rpc_connection_open_confirm(
        self,
        stream: 'grpclib.server.Stream[MsgConnectionOpenConfirm, MsgConnectionOpenConfirmResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.connection_open_confirm(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            '/ibc.core.connection.v1.Msg/ConnectionOpenInit': grpclib.const.Handler(
                self.__rpc_connection_open_init,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgConnectionOpenInit,
                MsgConnectionOpenInitResponse,
            ),
            '/ibc.core.connection.v1.Msg/ConnectionOpenTry': grpclib.const.Handler(
                self.__rpc_connection_open_try,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgConnectionOpenTry,
                MsgConnectionOpenTryResponse,
            ),
            '/ibc.core.connection.v1.Msg/ConnectionOpenAck': grpclib.const.Handler(
                self.__rpc_connection_open_ack,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgConnectionOpenAck,
                MsgConnectionOpenAckResponse,
            ),
            '/ibc.core.connection.v1.Msg/ConnectionOpenConfirm': grpclib.const.Handler(
                self.__rpc_connection_open_confirm,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgConnectionOpenConfirm,
                MsgConnectionOpenConfirmResponse,
            ),
        }



from __future__ import annotations

from typing import Any, Callable, TypeVar

_C = TypeVar('_C', bound=Callable[..., Any])


def _inherit(func: Callable[..., Any]) -> Callable[[_C], _C]:
    """Mark function as inheriting (via decorator) from ``func``.

    See `fix_altered_signature` method in ``conf.py`` for some context.
    """

    def decorator(inner: _C) -> _C:
        inner.__doc__ = func.__doc__
        inner.__name__ = func.__name__
        inner.__qualname__ = func.__qualname__
        inner.__module__ = func.__module__
        inner._inherit_signature_from_ = func  # type: ignore[attr-defined]

        return inner

    return decorator



from __future__ import annotations

import base64
from dataclasses import dataclass
from typing import Any, Final, Sequence

from eth_typing import HexStr
from eth_utils import keccak

from evmos.proto.autogen.py.cosmos.base.v1beta1 import Coin
from evmos.proto.autogen.py.cosmos.crypto import secp256k1 as secp
from evmos.proto.autogen.py.cosmos.tx import v1beta1 as tx
from evmos.proto.autogen.py.cosmos.tx.signing.v1beta1 import SignMode
from evmos.proto.autogen.py.ethermint.crypto.v1 import ethsecp256k1 as eth
from evmos.proto.utils import MessageGenerated, create_any_message

SIGN_DIRECT: Final = SignMode.SIGN_MODE_DIRECT
LEGACY_AMINO: Final = SignMode.SIGN_MODE_LEGACY_AMINO_JSON


@dataclass
class TxGeneratedSignInfo:
    """Type of :class:`TxGenerated` fields."""

    body: tx.TxBody
    """Transaction body."""
    auth_info: tx.AuthInfo
    """Transaction authentication info."""
    sign_bytes: HexStr
    """Raw bytes for signing as hex string."""


@dataclass
class TxGeneratedBase:
    """Transaction generated by this library."""

    sign_direct: TxGeneratedSignInfo
    """Data to sign directly (intended for keplr-style signing)."""
    legacy_amino: TxGeneratedSignInfo
    """Data to sign (inteded for use with EIP712 signing)."""


def create_body_with_multiple_messages(messages: Sequence[Any], memo: str) -> tx.TxBody:
    """Create a TxBody instance.

    Create :class:`~evmos.proto.autogen.py.cosmos.tx.v1beta1.TxBody`
    with multiple messages provided.
    """
    return tx.TxBody(messages=list(map(create_any_message, messages)), memo=memo)


def create_body(message: Any, memo: str) -> tx.TxBody:
    """Create a TxBody instance.

    Create :class:`~evmos.proto.autogen.py.cosmos.tx.v1beta1.TxBody`
    with single message provided.
    """
    return create_body_with_multiple_messages([message], memo)


def create_fee(fee: str, denom: str, gas_limit: int) -> tx.Fee:
    """Create a Fee instance.

    Create :class:`~evmos.proto.autogen.py.cosmos.tx.v1beta1.Fee`
    with provided parameters.
    """
    return tx.Fee(
        amount=[Coin(denom=denom, amount=fee)],
        gas_limit=gas_limit,
    )


def create_signer_info(
    algo: str,
    public_key: bytes,
    sequence: int,
    mode: int | SignMode,
) -> tx.SignerInfo:
    """Create a SignerInfo instance.

    Create :class:`~evmos.proto.autogen.py.cosmos.tx.v1beta1.SignerInfo`
    with provided parameters.
    """
    # NOTE: secp256k1 is going to be removed from evmos
    if algo == 'secp256k1':
        pubkey = MessageGenerated(
            message=secp.PubKey(key=public_key),
            path='cosmos.crypto.secp256k1.PubKey',
        )
    else:
        # NOTE: assume ethsecp256k1 by default because after mainnet is the only one
        # that is going to be supported
        pubkey = MessageGenerated(
            message=eth.PubKey(key=public_key),
            path='ethermint.crypto.v1.ethsecp256k1.PubKey',
        )

    return tx.SignerInfo(
        public_key=create_any_message(pubkey),
        mode_info=tx.ModeInfo(
            single=tx.ModeInfoSingle(mode=mode)  # type: ignore[arg-type]
        ),
        sequence=sequence,
    )


def create_auth_info(signer_info: tx.SignerInfo, fee: tx.Fee) -> tx.AuthInfo:
    """Create an AuthInfo instance.

    Create :class:`~evmos.proto.autogen.py.cosmos.tx.v1beta1.AuthInfo`
    with provided parameters.
    """
    return tx.AuthInfo(signer_infos=[signer_info], fee=fee)


def create_sig_doc(
    body_bytes: bytes, auth_info_bytes: bytes, chain_id: str, account_number: int
) -> tx.SignDoc:
    """Create a SignDoc instance.

    Create :class:`~evmos.proto.autogen.py.cosmos.tx.v1beta1.SignDoc`
    with provided parameters.
    """
    return tx.SignDoc(
        body_bytes=body_bytes,
        auth_info_bytes=auth_info_bytes,
        chain_id=chain_id,
        account_number=account_number,
    )


def create_transaction_with_multiple_messages(
    messages: Any,
    memo: str,
    fee: str,
    denom: str,
    gas_limit: int,
    algo: str,
    pub_key: str,
    sequence: int,
    account_number: int,
    chain_id: str,
) -> TxGeneratedBase:
    """Create transaction parameters with multiple messages."""
    body = create_body_with_multiple_messages(messages, memo)
    fee_message = create_fee(fee, denom, gas_limit)
    pub_key_decoded = base64.b64decode(pub_key)

    # AMINO
    sign_info_amino = create_signer_info(
        algo,
        bytes(pub_key_decoded),
        sequence,
        LEGACY_AMINO,
    )

    auth_info_amino = create_auth_info(sign_info_amino, fee_message)

    sign_doc_amino = create_sig_doc(
        bytes(body),
        bytes(auth_info_amino),
        chain_id,
        account_number,
    )

    to_sign_amino = keccak(bytes(sign_doc_amino))

    # SignDirect
    sign_info_direct = create_signer_info(
        algo,
        bytes(pub_key_decoded),
        sequence,
        SIGN_DIRECT,
    )

    auth_info_direct = create_auth_info(sign_info_direct, fee_message)

    sign_doc_direct = create_sig_doc(
        bytes(body),
        bytes(auth_info_direct),
        chain_id,
        account_number,
    )

    to_sign_direct = keccak(bytes(sign_doc_direct))

    return TxGeneratedBase(
        legacy_amino=TxGeneratedSignInfo(
            body=body,
            auth_info=auth_info_amino,
            sign_bytes=base64.b64encode(to_sign_amino).decode(),
        ),
        sign_direct=TxGeneratedSignInfo(
            body=body,
            auth_info=auth_info_direct,
            sign_bytes=base64.b64encode(to_sign_direct).decode(),
        ),
    )


def create_transaction(
    message: Any,
    memo: str,
    fee: str,
    denom: str,
    gas_limit: int,
    algo: str,
    pub_key: str,
    sequence: int,
    account_number: int,
    chain_id: str,
) -> TxGeneratedBase:
    """Create transaction parameters with a single message."""
    return create_transaction_with_multiple_messages(
        [message],
        memo,
        fee,
        denom,
        gas_limit,
        algo,
        pub_key,
        sequence,
        account_number,
        chain_id,
    )



from __future__ import annotations

from evmos.proto.autogen.py.cosmos.base.v1beta1 import Coin
from evmos.proto.autogen.py.cosmos.distribution.v1beta1 import (
    MsgWithdrawDelegatorReward,
    MsgWithdrawValidatorCommission,
)
from evmos.proto.autogen.py.cosmos.staking.v1beta1 import (
    MsgBeginRedelegate,
    MsgDelegate,
    MsgUndelegate,
)
from evmos.proto.utils import MessageGenerated


def create_msg_delegate(
    delegator_address: str,
    validator_address: str,
    amount: str,
    denom: str,
) -> MessageGenerated[MsgDelegate]:
    """Create a message for staking delegation."""
    value = Coin(denom=denom, amount=amount)

    message = MsgDelegate(
        delegator_address=delegator_address,
        validator_address=validator_address,
        amount=value,
    )

    return MessageGenerated(
        message=message,
        path='cosmos.staking.v1beta1.MsgDelegate',
    )


def create_msg_begin_redelegate(
    delegator_address: str,
    validator_src_address: str,
    validator_dst_address: str,
    amount: str,
    denom: str,
) -> MessageGenerated[MsgBeginRedelegate]:
    """Create a message for staking redelegation."""
    value = Coin(denom=denom, amount=amount)

    message = MsgBeginRedelegate(
        delegator_address=delegator_address,
        validator_src_address=validator_src_address,
        validator_dst_address=validator_dst_address,
        amount=value,
    )

    return MessageGenerated(
        message=message,
        path='cosmos.staking.v1beta1.MsgBeginRedelegate',
    )


def create_msg_undelegate(
    delegator_address: str,
    validator_address: str,
    amount: str,
    denom: str,
) -> MessageGenerated[MsgUndelegate]:
    """Create a message for staking undelegation."""
    value = Coin(denom=denom, amount=amount)

    message = MsgUndelegate(
        delegator_address=delegator_address,
        validator_address=validator_address,
        amount=value,
    )

    return MessageGenerated(
        message=message,
        path='cosmos.staking.v1beta1.MsgUndelegate',
    )


def create_msg_withdraw_delegator_reward(
    delegator_address: str,
    validator_address: str,
) -> MessageGenerated[MsgWithdrawDelegatorReward]:
    """Create a message for delegator rewards withdrawal."""
    message = MsgWithdrawDelegatorReward(
        delegator_address=delegator_address,
        validator_address=validator_address,
    )

    return MessageGenerated(
        message=message,
        path='cosmos.distribution.v1beta1.MsgWithdrawDelegatorReward',
    )


def create_msg_withdraw_validator_commission(
    validator_address: str,
) -> MessageGenerated[MsgWithdrawValidatorCommission]:
    """Create a message for validator commission withdrawal."""
    message = MsgWithdrawValidatorCommission(
        validator_address=validator_address,
    )

    return MessageGenerated(
        message=message,
        path='cosmos.distribution.v1beta1.MsgWithdrawValidatorCommission',
    )



from __future__ import annotations

from typing import Final, Sequence

from evmos.eip712.base import MsgInterface

MSG_CANCEL_FEE_SPLIT_TYPES: Final = {
    'MsgValue': [
        {'name': 'contract_address', 'type': 'string'},
        {'name': 'deployer_address', 'type': 'string'},
    ],
}
"""Types for message for fee split cancellation."""


def create_msg_cancel_fee_split(
    contract_address: str,
    deployer_address: str,
) -> MsgInterface:
    """Create message for fee split cancellation."""
    return {
        'type': 'evmos/MsgCancelFeeSplit',
        'value': {
            'contract_address': contract_address,
            'deployer_address': deployer_address,
        },
    }


MSG_REGISTER_FEE_SPLIT_TYPES: Final = {
    'MsgValue': [
        {'name': 'contract_address', 'type': 'string'},
        {'name': 'deployer_address', 'type': 'string'},
        {'name': 'withdrawer_address', 'type': 'string'},
        {'name': 'nonces', 'type': 'uint64[]'},
    ],
}
"""Types for message for fee split registration."""


def create_msg_register_fee_split(
    contract_address: str,
    deployer_address: str,
    withdrawer_address: str,
    nonces: Sequence[int],
) -> MsgInterface:
    """Create message for fee split registration."""
    return {
        'type': 'evmos/MsgRegisterFeeSplit',
        'value': {
            'contract_address': contract_address,
            'deployer_address': deployer_address,
            'withdrawer_address': withdrawer_address,
            'nonces': nonces,
        },
    }


MSG_UPDATE_FEE_SPLIT_TYPES: Final = {
    'MsgValue': [
        {'name': 'contract_address', 'type': 'string'},
        {'name': 'deployer_address', 'type': 'string'},
        {'name': 'withdrawer_address', 'type': 'string'},
    ],
}
"""Types for message for fee split update."""


def create_msg_update_fee_split(
    contract_address: str,
    deployer_address: str,
    withdrawer_address: str,
) -> MsgInterface:
    """Create message for fee split update."""
    return {
        'type': 'evmos/MsgUpdateFeeSplit',
        'value': {
            'contract_address': contract_address,
            'deployer_address': deployer_address,
            'withdrawer_address': withdrawer_address,
        },
    }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: evmos/feesplit/v1/feesplit.proto, evmos/feesplit/v1/genesis.proto, evmos/feesplit/v1/query.proto, evmos/feesplit/v1/tx.proto
# plugin: python-betterproto
# This file has been @generated
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase

from ....cosmos.base.query import v1beta1 as ___cosmos_base_query_v1_beta1__


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class FeeSplit(betterproto.Message):
    """
    FeeSplit defines an instance that organizes fee distribution conditions for
    the owner of a given smart contract
    """

    contract_address: str = betterproto.string_field(1)
    """hex address of registered contract"""

    deployer_address: str = betterproto.string_field(2)
    """bech32 address of contract deployer"""

    withdrawer_address: str = betterproto.string_field(3)
    """
    bech32 address of account receiving the transaction fees it defaults to
    deployer_address
    """


@dataclass(eq=False, repr=False)
class GenesisState(betterproto.Message):
    """GenesisState defines the module's genesis state."""

    params: 'Params' = betterproto.message_field(1)
    """module parameters"""

    fee_splits: List['FeeSplit'] = betterproto.message_field(2)
    """active registered contracts for fee distribution"""


@dataclass(eq=False, repr=False)
class Params(betterproto.Message):
    """Params defines the feesplit module params"""

    enable_fee_split: bool = betterproto.bool_field(1)
    """enable_fee_split defines a parameter to enable the feesplit module"""

    developer_shares: str = betterproto.string_field(2)
    """
    developer_shares defines the proportion of the transaction fees to be
    distributed to the registered contract owner
    """

    addr_derivation_cost_create: int = betterproto.uint64_field(3)
    """
    addr_derivation_cost_create defines the cost of address derivation for
    verifying the contract deployer at fee registration
    """


@dataclass(eq=False, repr=False)
class QueryFeeSplitsRequest(betterproto.Message):
    """
    QueryFeeSplitsRequest is the request type for the Query/FeeSplits RPC method.
    """

    pagination: '___cosmos_base_query_v1_beta1__.PageRequest' = (
        betterproto.message_field(1)
    )
    """pagination defines an optional pagination for the request."""


@dataclass(eq=False, repr=False)
class QueryFeeSplitsResponse(betterproto.Message):
    """
    QueryFeeSplitsResponse is the response type for the Query/FeeSplits RPC method.
    """

    fee_splits: List['FeeSplit'] = betterproto.message_field(1)
    pagination: '___cosmos_base_query_v1_beta1__.PageResponse' = (
        betterproto.message_field(2)
    )
    """pagination defines the pagination in the response."""


@dataclass(eq=False, repr=False)
class QueryFeeSplitRequest(betterproto.Message):
    """
    QueryFeeSplitRequest is the request type for the Query/FeeSplit RPC method.
    """

    contract_address: str = betterproto.string_field(1)
    """contract identifier is the hex contract address of a contract"""


@dataclass(eq=False, repr=False)
class QueryFeeSplitResponse(betterproto.Message):
    """
    QueryFeeSplitResponse is the response type for the Query/FeeSplit RPC method.
    """

    fee_split: 'FeeSplit' = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class QueryParamsRequest(betterproto.Message):
    """
    QueryParamsRequest is the request type for the Query/Params RPC method.
    """

    pass


@dataclass(eq=False, repr=False)
class QueryParamsResponse(betterproto.Message):
    """
    QueryParamsResponse is the response type for the Query/Params RPC method.
    """

    params: 'Params' = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class QueryDeployerFeeSplitsRequest(betterproto.Message):
    """
    QueryDeployerFeeSplitsRequest is the request type for the
    Query/DeployerFeeSplits RPC method.
    """

    deployer_address: str = betterproto.string_field(1)
    """deployer bech32 address"""

    pagination: '___cosmos_base_query_v1_beta1__.PageRequest' = (
        betterproto.message_field(2)
    )
    """pagination defines an optional pagination for the request."""


@dataclass(eq=False, repr=False)
class QueryDeployerFeeSplitsResponse(betterproto.Message):
    """
    QueryDeployerFeeSplitsResponse is the response type for the
    Query/DeployerFeeSplits RPC method.
    """

    contract_addresses: List[str] = betterproto.string_field(1)
    pagination: '___cosmos_base_query_v1_beta1__.PageResponse' = (
        betterproto.message_field(2)
    )
    """pagination defines the pagination in the response."""


@dataclass(eq=False, repr=False)
class QueryWithdrawerFeeSplitsRequest(betterproto.Message):
    """
    QueryWithdrawerFeeSplitsRequest is the request type for the
    Query/WithdrawerFeeSplits RPC method.
    """

    withdrawer_address: str = betterproto.string_field(1)
    """withdrawer bech32 address"""

    pagination: '___cosmos_base_query_v1_beta1__.PageRequest' = (
        betterproto.message_field(2)
    )
    """pagination defines an optional pagination for the request."""


@dataclass(eq=False, repr=False)
class QueryWithdrawerFeeSplitsResponse(betterproto.Message):
    """
    QueryWithdrawerFeeSplitsResponse is the response type for the
    Query/WithdrawerFeeSplits RPC method.
    """

    contract_addresses: List[str] = betterproto.string_field(1)
    pagination: '___cosmos_base_query_v1_beta1__.PageResponse' = (
        betterproto.message_field(2)
    )
    """pagination defines the pagination in the response."""


@dataclass(eq=False, repr=False)
class MsgRegisterFeeSplit(betterproto.Message):
    """MsgRegisterFeeSplit defines a message that registers a FeeSplit"""

    contract_address: str = betterproto.string_field(1)
    """contract hex address"""

    deployer_address: str = betterproto.string_field(2)
    """
    bech32 address of message sender, must be the same as the origin EOA
    sending the transaction which deploys the contract
    """

    withdrawer_address: str = betterproto.string_field(3)
    """bech32 address of account receiving the transaction fees"""

    nonces: List[int] = betterproto.uint64_field(4)
    """
    array of nonces from the address path, where the last nonce is the nonce
    that determines the contract's address - it can be an EOA nonce or a
    factory contract nonce
    """


@dataclass(eq=False, repr=False)
class MsgRegisterFeeSplitResponse(betterproto.Message):
    """
    MsgRegisterFeeSplitResponse defines the MsgRegisterFeeSplit response type
    """

    pass


@dataclass(eq=False, repr=False)
class MsgUpdateFeeSplit(betterproto.Message):
    """
    MsgUpdateFeeSplit defines a message that updates the withdrawer address for a
    registered FeeSplit
    """

    contract_address: str = betterproto.string_field(1)
    """contract hex address"""

    deployer_address: str = betterproto.string_field(2)
    """deployer bech32 address"""

    withdrawer_address: str = betterproto.string_field(3)
    """new withdrawer bech32 address for receiving the transaction fees"""


@dataclass(eq=False, repr=False)
class MsgUpdateFeeSplitResponse(betterproto.Message):
    """
    MsgUpdateFeeSplitResponse defines the MsgUpdateFeeSplit response type
    """

    pass


@dataclass(eq=False, repr=False)
class MsgCancelFeeSplit(betterproto.Message):
    """
    MsgCancelFeeSplit defines a message that cancels a registered FeeSplit
    """

    contract_address: str = betterproto.string_field(1)
    """contract hex address"""

    deployer_address: str = betterproto.string_field(2)
    """deployer bech32 address"""


@dataclass(eq=False, repr=False)
class MsgCancelFeeSplitResponse(betterproto.Message):
    """
    MsgCancelFeeSplitResponse defines the MsgCancelFeeSplit response type
    """

    pass


class QueryStub(betterproto.ServiceStub):
    async def fee_splits(
        self,
        query_fee_splits_request: 'QueryFeeSplitsRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryFeeSplitsResponse':
        return await self._unary_unary(
            '/evmos.feesplit.v1.Query/FeeSplits',
            query_fee_splits_request,
            QueryFeeSplitsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def fee_split(
        self,
        query_fee_split_request: 'QueryFeeSplitRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryFeeSplitResponse':
        return await self._unary_unary(
            '/evmos.feesplit.v1.Query/FeeSplit',
            query_fee_split_request,
            QueryFeeSplitResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def params(
        self,
        query_params_request: 'QueryParamsRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryParamsResponse':
        return await self._unary_unary(
            '/evmos.feesplit.v1.Query/Params',
            query_params_request,
            QueryParamsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def deployer_fee_splits(
        self,
        query_deployer_fee_splits_request: 'QueryDeployerFeeSplitsRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryDeployerFeeSplitsResponse':
        return await self._unary_unary(
            '/evmos.feesplit.v1.Query/DeployerFeeSplits',
            query_deployer_fee_splits_request,
            QueryDeployerFeeSplitsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def withdrawer_fee_splits(
        self,
        query_withdrawer_fee_splits_request: 'QueryWithdrawerFeeSplitsRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryWithdrawerFeeSplitsResponse':
        return await self._unary_unary(
            '/evmos.feesplit.v1.Query/WithdrawerFeeSplits',
            query_withdrawer_fee_splits_request,
            QueryWithdrawerFeeSplitsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class MsgStub(betterproto.ServiceStub):
    async def register_fee_split(
        self,
        msg_register_fee_split: 'MsgRegisterFeeSplit',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'MsgRegisterFeeSplitResponse':
        return await self._unary_unary(
            '/evmos.feesplit.v1.Msg/RegisterFeeSplit',
            msg_register_fee_split,
            MsgRegisterFeeSplitResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def update_fee_split(
        self,
        msg_update_fee_split: 'MsgUpdateFeeSplit',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'MsgUpdateFeeSplitResponse':
        return await self._unary_unary(
            '/evmos.feesplit.v1.Msg/UpdateFeeSplit',
            msg_update_fee_split,
            MsgUpdateFeeSplitResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def cancel_fee_split(
        self,
        msg_cancel_fee_split: 'MsgCancelFeeSplit',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'MsgCancelFeeSplitResponse':
        return await self._unary_unary(
            '/evmos.feesplit.v1.Msg/CancelFeeSplit',
            msg_cancel_fee_split,
            MsgCancelFeeSplitResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class QueryBase(ServiceBase):
    async def fee_splits(
        self, query_fee_splits_request: 'QueryFeeSplitsRequest'
    ) -> 'QueryFeeSplitsResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def fee_split(
        self, query_fee_split_request: 'QueryFeeSplitRequest'
    ) -> 'QueryFeeSplitResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def params(
        self, query_params_request: 'QueryParamsRequest'
    ) -> 'QueryParamsResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def deployer_fee_splits(
        self, query_deployer_fee_splits_request: 'QueryDeployerFeeSplitsRequest'
    ) -> 'QueryDeployerFeeSplitsResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def withdrawer_fee_splits(
        self, query_withdrawer_fee_splits_request: 'QueryWithdrawerFeeSplitsRequest'
    ) -> 'QueryWithdrawerFeeSplitsResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_fee_splits(
        self,
        stream: 'grpclib.server.Stream[QueryFeeSplitsRequest, QueryFeeSplitsResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.fee_splits(request)
        await stream.send_message(response)

    async def __rpc_fee_split(
        self,
        stream: 'grpclib.server.Stream[QueryFeeSplitRequest, QueryFeeSplitResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.fee_split(request)
        await stream.send_message(response)

    async def __rpc_params(
        self, stream: 'grpclib.server.Stream[QueryParamsRequest, QueryParamsResponse]'
    ) -> None:
        request = await stream.recv_message()
        response = await self.params(request)
        await stream.send_message(response)

    async def __rpc_deployer_fee_splits(
        self,
        stream: 'grpclib.server.Stream[QueryDeployerFeeSplitsRequest, QueryDeployerFeeSplitsResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.deployer_fee_splits(request)
        await stream.send_message(response)

    async def __rpc_withdrawer_fee_splits(
        self,
        stream: 'grpclib.server.Stream[QueryWithdrawerFeeSplitsRequest, QueryWithdrawerFeeSplitsResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.withdrawer_fee_splits(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            '/evmos.feesplit.v1.Query/FeeSplits': grpclib.const.Handler(
                self.__rpc_fee_splits,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryFeeSplitsRequest,
                QueryFeeSplitsResponse,
            ),
            '/evmos.feesplit.v1.Query/FeeSplit': grpclib.const.Handler(
                self.__rpc_fee_split,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryFeeSplitRequest,
                QueryFeeSplitResponse,
            ),
            '/evmos.feesplit.v1.Query/Params': grpclib.const.Handler(
                self.__rpc_params,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryParamsRequest,
                QueryParamsResponse,
            ),
            '/evmos.feesplit.v1.Query/DeployerFeeSplits': grpclib.const.Handler(
                self.__rpc_deployer_fee_splits,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryDeployerFeeSplitsRequest,
                QueryDeployerFeeSplitsResponse,
            ),
            '/evmos.feesplit.v1.Query/WithdrawerFeeSplits': grpclib.const.Handler(
                self.__rpc_withdrawer_fee_splits,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryWithdrawerFeeSplitsRequest,
                QueryWithdrawerFeeSplitsResponse,
            ),
        }


class MsgBase(ServiceBase):
    async def register_fee_split(
        self, msg_register_fee_split: 'MsgRegisterFeeSplit'
    ) -> 'MsgRegisterFeeSplitResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def update_fee_split(
        self, msg_update_fee_split: 'MsgUpdateFeeSplit'
    ) -> 'MsgUpdateFeeSplitResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def cancel_fee_split(
        self, msg_cancel_fee_split: 'MsgCancelFeeSplit'
    ) -> 'MsgCancelFeeSplitResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_register_fee_split(
        self,
        stream: 'grpclib.server.Stream[MsgRegisterFeeSplit, MsgRegisterFeeSplitResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.register_fee_split(request)
        await stream.send_message(response)

    async def __rpc_update_fee_split(
        self,
        stream: 'grpclib.server.Stream[MsgUpdateFeeSplit, MsgUpdateFeeSplitResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.update_fee_split(request)
        await stream.send_message(response)

    async def __rpc_cancel_fee_split(
        self,
        stream: 'grpclib.server.Stream[MsgCancelFeeSplit, MsgCancelFeeSplitResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.cancel_fee_split(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            '/evmos.feesplit.v1.Msg/RegisterFeeSplit': grpclib.const.Handler(
                self.__rpc_register_fee_split,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgRegisterFeeSplit,
                MsgRegisterFeeSplitResponse,
            ),
            '/evmos.feesplit.v1.Msg/UpdateFeeSplit': grpclib.const.Handler(
                self.__rpc_update_fee_split,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgUpdateFeeSplit,
                MsgUpdateFeeSplitResponse,
            ),
            '/evmos.feesplit.v1.Msg/CancelFeeSplit': grpclib.const.Handler(
                self.__rpc_cancel_fee_split,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgCancelFeeSplit,
                MsgCancelFeeSplitResponse,
            ),
        }
